"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const visitor_plugin_common_2 = require("@graphql-codegen/visitor-plugin-common");
const declaration_kinds_1 = require("./declaration-kinds");
const visitor_plugin_common_3 = require("@graphql-codegen/visitor-plugin-common");
const visitor_plugin_common_4 = require("@graphql-codegen/visitor-plugin-common");
const enum_values_1 = require("./enum-values");
class BaseTypesVisitor extends visitor_plugin_common_1.BaseVisitor {
    constructor(_schema, rawConfig, additionalConfig, defaultScalars = visitor_plugin_common_2.DEFAULT_SCALARS) {
        super(rawConfig, {
            addUnderscoreToArgsType: visitor_plugin_common_3.getConfigValue(rawConfig.addUnderscoreToArgsType, false),
            enumValues: enum_values_1.parseEnumValues(_schema, rawConfig.enumValues),
            declarationKind: declaration_kinds_1.normalizeDeclarationKind(rawConfig.declarationKind),
            ...additionalConfig,
        }, visitor_plugin_common_3.buildScalars(_schema, defaultScalars));
        this._schema = _schema;
        this._argumentsTransformer = new visitor_plugin_common_4.OperationVariablesToObject(this.scalars, this.convertName);
    }
    get scalarsDefinition() {
        const allScalars = Object.keys(this.config.scalars).map(scalarName => {
            const scalarValue = this.config.scalars[scalarName];
            const scalarType = this._schema.getType(scalarName);
            const comment = scalarType && scalarType.astNode && scalarType.description ? visitor_plugin_common_3.transformComment(scalarType.description, 1) : '';
            return comment + visitor_plugin_common_3.indent(`${scalarName}: ${scalarValue},`);
        });
        return new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind(this._parsedConfig.declarationKind.scalar)
            .withName('Scalars')
            .withComment('All built-in and custom scalars, mapped to their actual values')
            .withBlock(allScalars.join('\n')).string;
    }
    setDeclarationBlockConfig(config) {
        this._declarationBlockConfig = config;
    }
    setArgumentsTransformer(argumentsTransfomer) {
        this._argumentsTransformer = argumentsTransfomer;
    }
    NonNullType(node) {
        const asString = node.type;
        return asString;
    }
    InputObjectTypeDefinition(node) {
        return new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind(this._parsedConfig.declarationKind.input)
            .withName(this.convertName(node))
            .withComment(node.description)
            .withBlock(node.fields.join('\n')).string;
    }
    InputValueDefinition(node) {
        const comment = visitor_plugin_common_3.transformComment(node.description, 1);
        return comment + visitor_plugin_common_3.indent(`${node.name}: ${node.type},`);
    }
    Name(node) {
        return node.value;
    }
    FieldDefinition(node) {
        const typeString = node.type;
        const comment = visitor_plugin_common_3.transformComment(node.description, 1);
        return comment + visitor_plugin_common_3.indent(`${node.name}: ${typeString},`);
    }
    UnionTypeDefinition(node, key, parent) {
        const originalNode = parent[key];
        const possibleTypes = originalNode.types.map(t => (this.scalars[t.name.value] ? this._getScalar(t.name.value) : this.convertName(t))).join(' | ');
        return new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node))
            .withComment(node.description)
            .withContent(possibleTypes).string;
    }
    getObjectTypeDeclarationBlock(node, originalNode) {
        const optionalTypename = this.config.nonOptionalTypename ? '__typename' : '__typename?';
        const { type } = this._parsedConfig.declarationKind;
        const allFields = [...(this.config.addTypename ? [visitor_plugin_common_3.indent(`${optionalTypename}: '${node.name}'${type === 'class' ? ';' : ','}`)] : []), ...node.fields];
        const buildInterfaces = () => {
            if (!originalNode.interfaces || !node.interfaces.length) {
                return '';
            }
            const interfaces = originalNode.interfaces.map(i => this.convertName(i));
            if (type === 'interface' || type === 'class') {
                return ' extends ' + interfaces.join(', ') + (allFields.length ? ' ' : ' {}');
            }
            return interfaces.join(' & ') + (allFields.length ? ' & ' : '');
        };
        const interfaces = buildInterfaces();
        let declarationBlock = new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind(type)
            .withName(this.convertName(node))
            .withContent(interfaces)
            .withComment(node.description);
        return declarationBlock.withBlock(allFields.join('\n'));
    }
    ObjectTypeDefinition(node, key, parent) {
        const originalNode = parent[key];
        return [this.getObjectTypeDeclarationBlock(node, originalNode).string, this.buildArgumentsBlock(originalNode)].filter(f => f).join('\n\n');
    }
    getInterfaceTypeDeclarationBlock(node, originalNode) {
        const argumentsBlock = this.buildArgumentsBlock(originalNode);
        let declarationBlock = new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind(this._parsedConfig.declarationKind.interface)
            .withName(this.convertName(node))
            .withComment(node.description);
        return declarationBlock.withBlock(node.fields.join('\n'));
    }
    InterfaceTypeDefinition(node, key, parent) {
        const originalNode = parent[key];
        return [this.getInterfaceTypeDeclarationBlock(node, originalNode).string, this.buildArgumentsBlock(originalNode)].filter(f => f).join('\n\n');
    }
    ScalarTypeDefinition(node) {
        // We empty this because we handle scalars in a different way, see constructor.
        return '';
    }
    _buildEnumImport(identifier, source) {
        return `import { ${identifier} } from '${source}';`;
    }
    getEnumsImports() {
        return Object.keys(this.config.enumValues)
            .map(enumName => {
            const mappedValue = this.config.enumValues[enumName];
            if (mappedValue.sourceFile) {
                const identifier = mappedValue.sourceIdentifier !== mappedValue.typeIdentifier ? `${mappedValue.sourceIdentifier} as ${mappedValue.typeIdentifier}` : mappedValue.sourceIdentifier;
                return this._buildEnumImport(identifier, mappedValue.sourceFile);
            }
            return null;
        })
            .filter(a => a);
    }
    EnumTypeDefinition(node) {
        const enumName = node.name;
        // In case of mapped external enum string
        if (this.config.enumValues[enumName] && this.config.enumValues[enumName].sourceFile) {
            return null;
        }
        return new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('enum')
            .withName(this.convertName(node))
            .withComment(node.description)
            .withBlock(this.buildEnumValuesBlock(enumName, node.values)).string;
    }
    // We are using it in order to transform "description" field
    StringValue(node) {
        return node.value;
    }
    buildEnumValuesBlock(typeName, values) {
        return values
            .map(enumOption => {
            const optionName = this.convertName(enumOption, { useTypesPrefix: false, transformUnderscore: true });
            const comment = visitor_plugin_common_3.transformComment(enumOption.description, 1);
            let enumValue = enumOption.name;
            if (this.config.enumValues[typeName] && this.config.enumValues[typeName].mappedValues && this.config.enumValues[typeName].mappedValues[enumValue]) {
                enumValue = this.config.enumValues[typeName].mappedValues[enumValue];
            }
            return comment + visitor_plugin_common_3.indent(`${optionName}${this._declarationBlockConfig.enumNameValueSeparator} ${visitor_plugin_common_3.wrapWithSingleQuotes(enumValue)}`);
        })
            .join(',\n');
    }
    DirectiveDefinition(node) {
        return '';
    }
    buildArgumentsBlock(node) {
        const fieldsWithArguments = node.fields.filter(field => field.arguments && field.arguments.length > 0) || [];
        return fieldsWithArguments
            .map(field => {
            const name = node.name.value +
                (this.config.addUnderscoreToArgsType ? '_' : '') +
                this.convertName(field, {
                    useTypesPrefix: false,
                }) +
                'Args';
            return new visitor_plugin_common_3.DeclarationBlock(this._declarationBlockConfig)
                .export()
                .asKind(this._parsedConfig.declarationKind.arguments)
                .withName(this.convertName(name))
                .withComment(node.description)
                .withBlock(this._argumentsTransformer.transform(field.arguments)).string;
        })
            .join('\n\n');
    }
    _getScalar(name) {
        return `Scalars['${name}']`;
    }
    _getTypeForNode(node) {
        const typeAsString = node.name;
        if (this.scalars[typeAsString]) {
            return this._getScalar(typeAsString);
        }
        else if (this.config.enumValues[typeAsString]) {
            return this.config.enumValues[typeAsString].typeIdentifier;
        }
        return this.convertName(node);
    }
    NamedType(node) {
        return this._getTypeForNode(node);
    }
    ListType(node) {
        const asString = node.type;
        return this.wrapWithListType(asString);
    }
    SchemaDefinition() {
        return null;
    }
    wrapWithListType(str) {
        return `Array<${str}>`;
    }
}
exports.BaseTypesVisitor = BaseTypesVisitor;
//# sourceMappingURL=base-types-visitor.js.map