"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const autoBind = require("auto-bind");
const graphql_1 = require("graphql");
const typescript_variables_to_object_1 = require("./typescript-variables-to-object");
const base_types_visitor_1 = require("./base-types-visitor");
const MAYBE_REGEX = /^Maybe<(.*?)>$/;
const ARRAY_REGEX = /^Array<(.*?)>$/;
const GRAPHQL_TYPES = ['Query', 'Mutation', 'Subscription'];
class TsVisitor extends base_types_visitor_1.BaseTypesVisitor {
    constructor(schema, pluginConfig, additionalConfig = {}) {
        super(schema, pluginConfig, {
            avoidOptionals: pluginConfig.avoidOptionals || false,
            maybeValue: pluginConfig.maybeValue || 'T | null',
            constEnums: pluginConfig.constEnums || false,
            enumsAsTypes: pluginConfig.enumsAsTypes || false,
            immutableTypes: pluginConfig.immutableTypes || false,
            outputTypeGraphQL: pluginConfig.outputTypeGraphQL || false,
            ...(pluginConfig.outputTypeGraphQL
                ? {
                    declarationKind: {
                        type: 'class',
                        interface: 'abstract class',
                        arguments: 'type',
                        input: 'type',
                        scalar: 'type',
                    },
                }
                : {}),
            ...(additionalConfig || {}),
        });
        autoBind(this);
        this.setArgumentsTransformer(new typescript_variables_to_object_1.TypeScriptOperationVariablesToObject(this.scalars, this.convertName, this.config.avoidOptionals, this.config.immutableTypes));
        this.setDeclarationBlockConfig({
            enumNameValueSeparator: ' =',
        });
    }
    clearOptional(str) {
        if (str.startsWith('Maybe')) {
            return str.replace(MAYBE_REGEX, '$1');
        }
        return str;
    }
    NamedType(node) {
        return `Maybe<${super.NamedType(node)}>`;
    }
    ListType(node) {
        return `Maybe<${super.ListType(node)}>`;
    }
    wrapWithListType(str) {
        return `${this.config.immutableTypes ? 'ReadonlyArray' : 'Array'}<${str}>`;
    }
    NonNullType(node) {
        const baseValue = super.NonNullType(node);
        return this.clearOptional(baseValue);
    }
    ObjectTypeDefinition(node, key, parent) {
        const originalNode = parent[key];
        let declarationBlock = this.getObjectTypeDeclarationBlock(node, originalNode);
        if (this.config.outputTypeGraphQL && GRAPHQL_TYPES.indexOf(node.name) === -1) {
            // Add type-graphql ObjectType decorator
            const interfaces = originalNode.interfaces.map(i => this.convertName(i));
            let decoratorOptions = '';
            if (interfaces.length > 1) {
                decoratorOptions = `{ implements: [${interfaces.join(', ')}] }`;
            }
            else if (interfaces.length === 1) {
                decoratorOptions = `{ implements: ${interfaces[0]} }`;
            }
            declarationBlock = declarationBlock.withDecorator(`@TypeGraphQL.ObjectType(${decoratorOptions})`);
        }
        return [declarationBlock.string, this.buildArgumentsBlock(originalNode)].filter(f => f).join('\n\n');
    }
    InterfaceTypeDefinition(node, key, parent) {
        const originalNode = parent[key];
        let declarationBlock = this.getInterfaceTypeDeclarationBlock(node, originalNode);
        if (this.config.outputTypeGraphQL) {
            declarationBlock = declarationBlock.withDecorator('@TypeGraphQL.InterfaceType()');
        }
        return [declarationBlock.string, this.buildArgumentsBlock(originalNode)].filter(f => f).join('\n\n');
    }
    parseType(typeString) {
        const isNullable = !!typeString.match(MAYBE_REGEX);
        const nonNullableType = typeString.replace(MAYBE_REGEX, '$1');
        const isArray = !!nonNullableType.match(ARRAY_REGEX);
        const singularType = nonNullableType.replace(ARRAY_REGEX, '$1');
        const isScalar = singularType.match(/Scalars\['(.*?)'\]$/);
        const type = singularType.replace(/Scalars\['(.*?)'\]$/, (match, type) => (global[type] ? type : `TypeGraphQL.${type}`));
        return { type, isNullable, isArray, isScalar };
    }
    FieldDefinition(node, key, parent) {
        let typeString = node.type;
        const originalFieldNode = parent[key];
        const addOptionalSign = !this.config.avoidOptionals && originalFieldNode.type.kind !== graphql_1.Kind.NON_NULL_TYPE;
        const comment = visitor_plugin_common_1.transformComment(node.description, 1);
        let decorator = '';
        if (super.config.outputTypeGraphQL) {
            const type = this.parseType(typeString);
            decorator = '\n' + visitor_plugin_common_1.indent(`@TypeGraphQL.Field(type => ${type.isArray ? `[${type.type}]` : type.type}${type.isNullable ? ', { nullable: true }' : ''})`) + '\n';
            if (!type.isScalar && !type.isArray && !type.isNullable) {
                typeString = `FixDecoratorIssue<${typeString}>`;
            }
        }
        return comment + decorator + visitor_plugin_common_1.indent(`${this.config.immutableTypes ? 'readonly ' : ''}${node.name}${super.config.outputTypeGraphQL ? '!' : addOptionalSign ? '?' : ''}: ${typeString}${super.config.outputTypeGraphQL ? ';' : ','}`);
    }
    InputValueDefinition(node, key, parent) {
        const originalFieldNode = parent[key];
        const addOptionalSign = !this.config.avoidOptionals && originalFieldNode.type.kind !== graphql_1.Kind.NON_NULL_TYPE;
        const comment = visitor_plugin_common_1.transformComment(node.description, 1);
        return comment + visitor_plugin_common_1.indent(`${this.config.immutableTypes ? 'readonly ' : ''}${node.name}${addOptionalSign ? '?' : ''}: ${node.type},`);
    }
    EnumTypeDefinition(node) {
        const enumName = node.name;
        // In case of mapped external enum string
        if (this.config.enumValues[enumName] && this.config.enumValues[enumName].sourceFile) {
            return null;
        }
        let declaration;
        if (this.config.enumsAsTypes) {
            declaration = new visitor_plugin_common_1.DeclarationBlock(this._declarationBlockConfig)
                .export()
                .asKind('type')
                .withComment(node.description)
                .withName(this.convertName(node))
                .withContent('\n' +
                node.values
                    .map(enumOption => {
                    let enumValue = enumOption.name;
                    const comment = visitor_plugin_common_1.transformComment(enumOption.description, 1);
                    if (this.config.enumValues[enumName] && this.config.enumValues[enumName].mappedValues && this.config.enumValues[enumName].mappedValues[enumValue]) {
                        enumValue = this.config.enumValues[enumName].mappedValues[enumValue];
                    }
                    return comment + visitor_plugin_common_1.indent(visitor_plugin_common_1.wrapWithSingleQuotes(enumValue));
                })
                    .join(' |\n')).string;
        }
        else {
            declaration = new visitor_plugin_common_1.DeclarationBlock(this._declarationBlockConfig)
                .export()
                .asKind(this.config.constEnums ? 'const enum' : 'enum')
                .withName(this.convertName(node))
                .withComment(node.description)
                .withBlock(this.buildEnumValuesBlock(enumName, node.values)).string;
        }
        if (super.config.outputTypeGraphQL) {
            declaration = declaration + `TypeGraphQL.registerEnumType(${this.convertName(node)}, { name: '${this.convertName(node)}' });\n`;
        }
        return declaration;
    }
}
exports.TsVisitor = TsVisitor;
//# sourceMappingURL=visitor.js.map