import { parse, printSchema, visit, TypeInfo, visitWithTypeInfo, getNamedType, isIntrospectionType, printIntrospectionSchema, isObjectType } from 'graphql';
import { TsVisitor } from './visitor';
import { TsIntrospectionVisitor } from './introspection-visitor';
export * from './typescript-variables-to-object';
export * from './visitor';
const TYPE_GRAPHQL_IMPORT = `import * as TypeGraphQL from 'type-graphql';`;
const FIX_DECORATOR_ISSUE_TYPE = `type FixDecoratorIssue<T> = T`;
const isDefinitionInterface = (definition) => definition.indexOf('@TypeGraphQL.InterfaceType()') !== -1;
export const plugin = (schema, documents, config) => {
    const visitor = new TsVisitor(schema, config);
    const printedSchema = printSchema(schema);
    const astNode = parse(printedSchema);
    const maybeValue = `export type Maybe<T> = ${visitor.config.maybeValue};`;
    const visitorResult = visit(astNode, { leave: visitor });
    const introspectionDefinitions = includeIntrospectionDefinitions(schema, documents, config);
    const scalars = visitor.scalarsDefinition;
    let definitions = visitorResult.definitions;
    if (config.outputTypeGraphQL) {
        // Sort output by interfaces first, classes last to prevent TypeScript errors
        definitions.sort((definition1, definition2) => (isDefinitionInterface(definition1) ? -1 : 1));
    }
    return {
        prepend: [...visitor.getEnumsImports(), maybeValue, ...(config.outputTypeGraphQL ? [TYPE_GRAPHQL_IMPORT, FIX_DECORATOR_ISSUE_TYPE] : [])],
        content: [scalars, ...definitions, ...introspectionDefinitions].join('\n'),
    };
};
function includeIntrospectionDefinitions(schema, documents, config) {
    const typeInfo = new TypeInfo(schema);
    const usedTypes = [];
    const documentsVisitor = visitWithTypeInfo(typeInfo, {
        Field() {
            const type = getNamedType(typeInfo.getType());
            if (isIntrospectionType(type) && !usedTypes.includes(type)) {
                usedTypes.push(type);
            }
        },
    });
    documents.forEach(doc => visit(doc.content, documentsVisitor));
    const typesToInclude = [];
    usedTypes.forEach(type => {
        collectTypes(type);
    });
    const visitor = new TsIntrospectionVisitor(schema, config, typesToInclude);
    const result = visit(parse(printIntrospectionSchema(schema)), { leave: visitor });
    // recursively go through each `usedTypes` and their children and collect all used types
    // we don't care about Interfaces, Unions and others, but Objects and Enums
    function collectTypes(type) {
        if (typesToInclude.includes(type)) {
            return;
        }
        typesToInclude.push(type);
        if (isObjectType(type)) {
            const fields = type.getFields();
            Object.keys(fields).forEach(key => {
                const field = fields[key];
                const type = getNamedType(field.type);
                collectTypes(type);
            });
        }
    }
    return result.definitions;
}
export { TsVisitor, TsIntrospectionVisitor };
//# sourceMappingURL=index.js.map